#include <QApplication>
#include <QWidget>
#include <QLabel>
#include <QStyle>
#include <QDebug>

#include <QFrame>
#include <QStringList>
#include <QVBoxLayout>

#include <algorithm>

static struct {
    const QString name;
    const QFrame::Shape shape;
    const QFrame::Shadow shadow;
} frames[] = {
    {"QFrame::Box, QFrame::Plain", QFrame::Box, QFrame::Plain},
    {"QFrame::Box, QFrame::Raised", QFrame::Box, QFrame::Raised},
    {"QFrame::Box, QFrame::Sunken", QFrame::Box, QFrame::Sunken},
    {"QFrame::Panel, QFrame::Raised", QFrame::Panel, QFrame::Raised},
    {"QFrame::Panel, QFrame::Sunken", QFrame::Panel, QFrame::Sunken}
};

QWidget *createFrame(int i)
{
    QFrame *newFrame = new QFrame();
    QVBoxLayout *vl = new QVBoxLayout(newFrame);
    vl->addWidget(new QLabel(frames[i].name));
    newFrame->setFrameShape(frames[i].shape);
    newFrame->setFrameShadow(frames[i].shadow);
    return newFrame;
}

QString scalingDescription()
{
    QString r = QLatin1String(QT_VERSION_STR) + QLatin1Char(' ')  + QGuiApplication::platformName()
        + QLatin1Char(' ')  + qApp->style()->objectName()
        +  QLatin1String(" dpr=")  + QString::number(qApp->devicePixelRatio());
    return r;
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

#if QT_VERSION >= 0x050600
    if (args.contains("-n")) {
        qDebug() << "DisableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
    } else   {
        qDebug() << "EnableHighDpiScaling";
        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    }
#endif

    QApplication a(argc, argv);

    int idx = args.indexOf("-i");
    if (idx >= 0)
        idx = args.at(idx + 1).toInt();

    QWidget main;
    QString t = scalingDescription();
    qDebug() << t;    

    main.setWindowTitle(QT_VERSION_STR);
    QVBoxLayout *mainL = new QVBoxLayout(&main);
    mainL->addWidget(new QLabel(t));

    int size = int(sizeof(frames)/sizeof(frames[0]));
    if (idx >= 0 && idx < size) {
        mainL->addWidget(createFrame(idx));
    } else {
        for (int i =0; i < size;++i)
            mainL->addWidget(createFrame(i));
    }

    main.show();

    return a.exec();
}
